use nix::unistd;
use std::boxed::Box;
use std::fs::File;
use std::io::*;
use tokio::time::{sleep, Duration};

pub fn read_lines(filename: &String, offset: u64) -> Option<Box<dyn Read>> {
    let mut file = match File::open(filename) {
        Ok(o) => o,
        Err(e) => {
            println!("error: {e}");
            return None;
        }
    };
    file.seek(SeekFrom::Start(offset)).unwrap();
    let lines: Box<dyn Read> = Box::new(BufReader::new(file));
    Some(lines)
}

pub async fn sleep_s(s: u64) {
    sleep(Duration::from_secs(s)).await;
}

#[allow(dead_code)]
pub async fn sleep_ms(m: u64) {
    sleep(Duration::from_millis(m)).await;
}

pub fn gethostname(show_fqdn: bool) -> String {
    let hostname_cstr = unistd::gethostname().expect("Failed getting hostname");
    let fqdn = hostname_cstr
        .to_str()
        .expect("Hostname wasn't valid UTF-8")
        .to_string();
    let hostname: Vec<&str> = fqdn.split(".").collect();
    if show_fqdn {
        return fqdn;
    }
    hostname[0].to_string()
}
